@@ -3,28 +3,28 @@
 import sys
 from typing import Dict, Optional, Tuple
 
-import attr
 import sympy as sp
-from attr.validators import instance_of
+from attrs import field, frozen
+from attrs.validators import instance_of
 from qrules.particle import Particle
 
 from . import (
     BlattWeisskopfSquared,
     BreakupMomentumSquared,
+    EnergyDependentWidth,
     PhaseSpaceFactor,
     PhaseSpaceFactorAnalytic,
     PhaseSpaceFactorProtocol,
     relativistic_breit_wigner,
-    relativistic_breit_wigner_with_ff,
 )
 
 if sys.version_info >= (3, 8):
     from typing import Protocol
-else:
+else:  # pragma: no cover
     from typing_extensions import Protocol
 
 
-@attr.frozen
+@frozen
 class TwoBodyKinematicVariableSet:
     """Data container for the essential variables of a two-body decay.
 
@@ -34,12 +34,12 @@ class TwoBodyKinematicVariableSet:
     amplitude model.
     """
 
-    incoming_state_mass: sp.Symbol = attr.ib(validator=instance_of(sp.Symbol))
-    outgoing_state_mass1: sp.Symbol = attr.ib(validator=instance_of(sp.Symbol))
-    outgoing_state_mass2: sp.Symbol = attr.ib(validator=instance_of(sp.Symbol))
-    helicity_theta: sp.Symbol = attr.ib(validator=instance_of(sp.Symbol))
-    helicity_phi: sp.Symbol = attr.ib(validator=instance_of(sp.Symbol))
-    angular_momentum: Optional[int] = attr.ib(default=None)
+    incoming_state_mass: sp.Symbol = field(validator=instance_of(sp.Symbol))
+    outgoing_state_mass1: sp.Symbol = field(validator=instance_of(sp.Symbol))
+    outgoing_state_mass2: sp.Symbol = field(validator=instance_of(sp.Symbol))
+    helicity_theta: sp.Symbol = field(validator=instance_of(sp.Symbol))
+    helicity_phi: sp.Symbol = field(validator=instance_of(sp.Symbol))
+    angular_momentum: Optional[int] = field(default=None)
 
 
 BuilderReturnType = Tuple[sp.Expr, Dict[sp.Symbol, float]]
@@ -93,14 +93,14 @@ def create_non_dynamic_with_ff(
             " factor!"
         )
     q_squared = BreakupMomentumSquared(
-        s=variable_pool.incoming_state_mass ** 2,
+        s=variable_pool.incoming_state_mass**2,
         m_a=variable_pool.outgoing_state_mass1,
         m_b=variable_pool.outgoing_state_mass2,
     )
     meson_radius = sp.Symbol(f"d_{resonance.name}")
     form_factor_squared = BlattWeisskopfSquared(
         angular_momentum,
-        z=q_squared * meson_radius ** 2,
+        z=q_squared * meson_radius**2,
     )
     return (
         sp.sqrt(form_factor_squared),
@@ -116,41 +116,58 @@ class RelativisticBreitWignerBuilder:
     :meth:`.set_dynamics`.
 
     Args:
-        form_factor: Formulate a relativistic Breit-Wigner function with form
-            factor, using :func:`.relativistic_breit_wigner_with_ff`. If set to
-            `False`, :meth:`__call__` builds a
-            :func:`.relativistic_breit_wigner` (_without_ form factor).
+        form_factor: Formulate a relativistic Breit-Wigner function multiplied
+            by a Blatt-Weisskopf form factor, see Equation
+            :eq:`BlattWeisskopfSquared`.
+        energy_dependent_width: Use an `.EnergyDependentWidth` in the
+            denominator of the Breit-Wigner. See
+            :pdg-review:`2020; Resonances; p.6`, Equation (49.21).
         phsp_factor: A class that complies with the
-            `.PhaseSpaceFactorProtocol`. Defaults to `.PhaseSpaceFactor`.
+            `.PhaseSpaceFactorProtocol` that is used in the energy-dependent
+            width. Defaults to `.PhaseSpaceFactor`.
     """
 
     def __init__(
         self,
         form_factor: bool = False,
+        energy_dependent_width: bool = False,
         phsp_factor: Optional[PhaseSpaceFactorProtocol] = None,
     ) -> None:
         if phsp_factor is None:
             phsp_factor = PhaseSpaceFactor
-        self.__phsp_factor = phsp_factor
-        self.__with_form_factor = form_factor
+        self.phsp_factor = phsp_factor
+        self.energy_dependent_width = energy_dependent_width
+        self.form_factor = form_factor
 
     def __call__(
         self, resonance: Particle, variable_pool: TwoBodyKinematicVariableSet
     ) -> "BuilderReturnType":
         """Formulate a relativistic Breit-Wigner for this resonance."""
-        if self.__with_form_factor:
-            return self.__formulate_with_form_factor(resonance, variable_pool)
-        return self.__formulate(resonance, variable_pool)
+        if self.energy_dependent_width:
+            expr, parameter_defaults = self.__energy_dependent_breit_wigner(
+                resonance, variable_pool
+            )
+        else:
+            expr, parameter_defaults = self.__simple_breit_wigner(
+                resonance, variable_pool
+            )
+        if self.form_factor:
+            form_factor, parameters = self.__create_form_factor(
+                resonance, variable_pool
+            )
+            parameter_defaults.update(parameters)
+            return form_factor * expr, parameter_defaults
+        return expr, parameter_defaults
 
     @staticmethod
-    def __formulate(
+    def __simple_breit_wigner(
         resonance: Particle, variable_pool: TwoBodyKinematicVariableSet
     ) -> "BuilderReturnType":
         inv_mass = variable_pool.incoming_state_mass
         res_mass = sp.Symbol(f"m_{resonance.name}")
         res_width = sp.Symbol(f"Gamma_{resonance.name}")
         expression = relativistic_breit_wigner(
-            s=inv_mass ** 2,
+            s=inv_mass**2,
             mass0=res_mass,
             gamma0=res_width,
         )
@@ -160,7 +177,7 @@ class RelativisticBreitWignerBuilder:
         }
         return expression, parameter_defaults
 
-    def __formulate_with_form_factor(
+    def __energy_dependent_breit_wigner(
         self, resonance: Particle, variable_pool: TwoBodyKinematicVariableSet
     ) -> "BuilderReturnType":
         if variable_pool.angular_momentum is None:
@@ -170,29 +187,69 @@ class RelativisticBreitWignerBuilder:
             )
 
         inv_mass = variable_pool.incoming_state_mass
-        res_mass = sp.Symbol(f"m_{resonance.name}")
-        res_width = sp.Symbol(f"Gamma_{resonance.name}")
-        product1_inv_mass = variable_pool.outgoing_state_mass1
-        product2_inv_mass = variable_pool.outgoing_state_mass2
+        m_a = variable_pool.outgoing_state_mass1
+        m_b = variable_pool.outgoing_state_mass2
         angular_momentum = variable_pool.angular_momentum
-        meson_radius = sp.Symbol(f"d_{resonance.name}")
+        res_mass, res_width, meson_radius = self.__create_symbols(resonance)
 
-        expression = relativistic_breit_wigner_with_ff(
-            s=inv_mass ** 2,
+        s = inv_mass**2
+        mass_dependent_width = EnergyDependentWidth(
+            s=s,
             mass0=res_mass,
             gamma0=res_width,
-            m_a=product1_inv_mass,
-            m_b=product2_inv_mass,
+            m_a=m_a,
+            m_b=m_b,
             angular_momentum=angular_momentum,
             meson_radius=meson_radius,
-            phsp_factor=self.__phsp_factor,
+            phsp_factor=self.phsp_factor,
+        )
+        breit_wigner_expr = (res_mass * res_width) / (
+            res_mass**2 - s - mass_dependent_width * res_mass * sp.I
         )
         parameter_defaults = {
             res_mass: resonance.mass,
             res_width: resonance.width,
             meson_radius: 1,
         }
-        return expression, parameter_defaults
+        return breit_wigner_expr, parameter_defaults
+
+    def __create_form_factor(
+        self, resonance: Particle, variable_pool: TwoBodyKinematicVariableSet
+    ) -> "BuilderReturnType":
+        if variable_pool.angular_momentum is None:
+            raise ValueError(
+                "Angular momentum is not defined but is required in the"
+                " form factor!"
+            )
+
+        inv_mass = variable_pool.incoming_state_mass
+        angular_momentum = variable_pool.angular_momentum
+        res_mass, res_width, meson_radius = self.__create_symbols(resonance)
+
+        q_squared = BreakupMomentumSquared(
+            s=inv_mass**2,
+            m_a=variable_pool.outgoing_state_mass1,
+            m_b=variable_pool.outgoing_state_mass2,
+        )
+        ff_squared = BlattWeisskopfSquared(
+            angular_momentum, z=q_squared * meson_radius**2
+        )
+        form_factor = sp.sqrt(ff_squared)
+        parameter_defaults = {
+            res_mass: resonance.mass,
+            res_width: resonance.width,
+            meson_radius: 1,
+        }
+        return form_factor, parameter_defaults
+
+    @staticmethod
+    def __create_symbols(
+        resonance: Particle,
+    ) -> Tuple[sp.Symbol, sp.Symbol, sp.Symbol]:
+        res_mass = sp.Symbol(f"m_{resonance.name}")
+        res_width = sp.Symbol(f"Gamma_{resonance.name}")
+        meson_radius = sp.Symbol(f"d_{resonance.name}")
+        return res_mass, res_width, meson_radius
 
 
 create_relativistic_breit_wigner = RelativisticBreitWignerBuilder(
@@ -206,6 +263,7 @@ form factor.
 """
 
 create_relativistic_breit_wigner_with_ff = RelativisticBreitWignerBuilder(
+    energy_dependent_width=True,
     form_factor=True,
     phsp_factor=PhaseSpaceFactor,
 ).__call__
@@ -217,6 +275,7 @@ form factor and a 'normal' `.PhaseSpaceFactor`.
 """
 
 create_analytic_breit_wigner = RelativisticBreitWignerBuilder(
+    energy_dependent_width=True,
     form_factor=True,
     phsp_factor=PhaseSpaceFactorAnalytic,
 ).__call__
