@@ -8,6 +8,7 @@ import qrules
 from _pytest.config import Config
 from _pytest.fixtures import SubRequest
 from qrules import ParticleCollection, ReactionInfo, load_default_particles
+from qrules.settings import NumberOfThreads
 
 from ampform import get_builder
 from ampform.dynamics.builder import create_relativistic_breit_wigner_with_ff
@@ -15,6 +16,10 @@ from ampform.helicity import HelicityModel
 
 logging.getLogger().setLevel(level=logging.ERROR)
 
+# Ensure consistent test coverage when running pytest multithreaded
+# https://github.com/ComPWA/qrules/issues/11
+NumberOfThreads.set(1)
+
 
 @pytest.fixture(scope="session")
 def particle_database() -> ParticleCollection:
